@@ -110,10 +110,53 @@ class Paper_Plot:
             transform=ax.transAxes
         )
 
+    def crop_to_content(
+        self, image, background_color=None, replace_background_color=None
+    ):
+        N_CHANNELS = image.shape[-1]  # Number of channels in the picture
+        image_shape = image.shape
+        n_pixels = image.shape[0] * image.shape[1]
+
+        img_flatten = image.reshape((n_pixels, N_CHANNELS))
+
+        # If no background color is specified, we take the most frequent color
+        if background_color is None:
+            unique, counts = np.unique(img_flatten, return_counts=True, axis=0)
+            idx_max_count = np.argmax(counts)
+            most_frequent_color = unique[idx_max_count]
+            background_color = most_frequent_color
+
+        # I think I know why this works
+        indices = np.argwhere(np.any(image[:, :] != background_color, axis=2))
+
+        lower_height = np.min(indices[:, 0])
+        upper_height = np.max(indices[:, 0])
+        lower_width = np.min(indices[:, 1])
+        upper_width = np.max(indices[:, 1])
+
+        if not replace_background_color is None:
+            N_CHANNELS_BG = len(replace_background_color)
+
+            # If the background color has more channels than the picture, we need to introduce the alpha channel
+            if N_CHANNELS_BG > N_CHANNELS and replace_background_color[-1] != 1.0:
+                image_copy = np.ones(shape=(image_shape[0], image_shape[1], 4))
+                image_copy[:, :, :3] = image
+                image = image_copy
+                background_color = [
+                    *background_color,
+                    1.0,
+                ]  # Extend the background color with the alpha channel
+            elif N_CHANNELS_BG < N_CHANNELS:
+                replace_background_color = [*replace_background_color, 1.0]
+
+            indices = np.argwhere(np.all(image[:, :] == background_color, axis=2))
+            image[indices[:, 0], indices[:, 1], :] = replace_background_color
+
+        return image[lower_height:upper_height, lower_width:upper_width, :]
+
     def crop(self, image, width, height=None):
         if height is None:
-            height = image.shape[1]
-
+            height = image.shape[0]
         o = [int(image.shape[0] / 2), int(image.shape[1] / 2)]
         lower_height = o[0] - int(height / 2)
         lower_width = o[1] - int(width / 2)
@@ -135,17 +178,31 @@ class Paper_Plot:
             s.set_visible(False)
 
     def spine_axis(
-        self, subplotspec, color="black", which=["left", "right", "top", "bottom"]
+        self,
+        spec,
+        color="black",
+        which=["left", "right", "top", "bottom"],
+        zorder=2,
+        label="spine",
     ):
-        a = self._fig.add_axes(subplotspec.get_position(self._fig), zorder=99)
+        try:
+            a = self._fig.add_axes(
+                spec.get_position(self._fig), zorder=zorder, label=label
+            )
+        except:
+            a = self._fig.add_axes(spec, zorder=zorder, label=label)
+
         a.set_facecolor([0, 0, 0, 0])
         a.tick_params(
             axis="both", which="both", bottom=0, left=0, labelbottom=0, labelleft=0
         )
-        for k in which:
+        for k in ["left", "right", "top", "bottom"]:
             s = a.spines[k]
-            s.set_visible(True)
-            s.set_color(color)
+            if k in which:
+                s.set_visible(True)
+                s.set_color(color)
+            else:
+                s.set_visible(False)
         return a
 
     def row(self, row_idx, sl=slice(None, None, None), gs=None):
@@ -395,6 +452,51 @@ def gradient_line(ax, x, y, c, lw=2.0, cmap="viridis", c_norm=None, n_inter=10):
         pass
 
 
+def get_rgba_colors_red_blue(spins, opacity=1.0, cardinal=np.array([0, 0, 1])):
+    cardinal = np.asarray(cardinal)
+
+    def mix(x, y, a):
+        return x * (1.0 - a) + y * a
+
+    red = np.array([1, 0, 0, opacity])
+    blue = np.array([0, 0, 1, opacity])
+
+    rgba = []
+
+    for spin in spins:
+        projection_z = cardinal.dot(spin)
+        _rgba = mix(red, blue, (projection_z + 1.0) / 2.0)
+        rgba.append([_rgba])
+
+    return rgba
+
+
+def get_rgba_colors_red_green_blue(spins, opacity=1.0, cardinal=np.array([0, 0, 1])):
+    cardinal = np.asarray(cardinal)
+
+    def mix(x, y, a):
+        return x * (1.0 - a) + y * a
+
+    red = np.array([1, 0, 0, opacity])
+    green = np.array([0, 1, 0, opacity])
+    blue = np.array([0, 0, 1, opacity])
+
+    rgba = []
+
+    for spin in spins:
+        projection_z = cardinal.dot(spin)
+
+        if projection_z < 0:
+            _rgba = mix(green, red, -projection_z)
+
+        if projection_z >= 0:
+            _rgba = mix(green, blue, projection_z)
+
+        rgba.append([_rgba])
+
+    return rgba
+
+
 def get_rgba_colors(
     spins,
     opacity=1.0,
